\MANDEL.XPL	APR-27-99
\Mandelbrot Set Plotter
\Loren Blaney
\
\ The Mandelbrot set is the set of points C that do not cause Z to go to
\ infinity when this equation is repeatedly calculated:
\
\	Z := Z^2 + C
\
\ Z and C are complex numbers (Z^2 is Z squared). They consist of two values
\ called real and imaginary, which are indicated like this:
\
\	Z  =  Zr + Zi*i
\	C  =  Cr + Ci*i
\
\	Z^2  =  (Zr + Zi*i)^2  =  Zr^2 + 2*Zr*Zi*i + Zi^2*i^2
\
\ By definition i^2 = -1, thus:
\
\	Z^2  =  Zr^2 - Zi^2 + 2*Zr*Zi*i
\
\ Keeping the real and imaginary parts separate, the original equation becomes:
\
\	Zr':= Zr^2 - Zi^2 + Cr
\	Zi := 2*Zr*Zi + Ci
\	Zr := Zr'
\ (Zr' is simply to keep from changing Zr prematurely.)
\
\ Plotting the points C in the Mandelbrot set gives the characteristic snowman
\ shape. The real part (Cr) is plotted on the X axis, and the imaginary part
\ (Ci) is plotted on the Y axis. If colors are plotted according to the rate
\ that Z approaches infinity, beautiful patterns emerge.


inc	\CXPL\CODESI;	\Intrinsic definitions
int	X, Y,		\Screen coordinates of current point
	Count;		\Iteration counter
real	Cr, Ci,		\Coordinates scaled to +/-2 range
	Zr, Zi,		\Complex accumulator
	ZrZr, ZiZi,	\Complex accumulator squared
	Temp;		\Temporary scratch
begin
SetVid($13);		\Set 320x200x256 graphic screen

for Y:= 0, 100-1 do			\For the top half of the screen...
    for X:= 0, 320-1 do			    \For all the points horizontally...
	begin
	Cr:= (Float(X)/320.0 - 0.5) * 4.0;	\Range -2.0 to +2.0
	Ci:= (Float(Y)/100.0) * 1.5;		\Range  0.0 to +1.5

	Count:= 0;   Zr:= 0.0;   Zi:= 0.0;
	loop	begin
		ZrZr:= Zr * Zr;
		ZiZi:= Zi * Zi;

		if ZrZr + ZiZi > 2.0 then quit;	\Heads toward infinity

		Temp:= Zi * Zr;		\(Rearranged to speed up calculations)
		Zr:= ZrZr - ZiZi + Cr;
		Zi:= Temp + Temp + Ci;

		Count:= Count +1;
		if Count >= 64 then quit;	\Quit if in the Mandelbrot set
		end;				\(64 is close enough to infinity)

	if Count < 64 then			\Leave points in Mandel set black
		begin				\ otherwise plot a pretty color
		Point(X, 100-Y, Count&$0F);	\Double speed by taking advantage
		Point(X, 100+Y, Count&$0F);	\ of symmetry about X axis
		end;
	end;

Openi(0);		\Wait for keystroke, so plot can be admired
if Chin(1) then;
SetVid($03);		\Restore standard text display
end;
